#!perl
use strict;
use POSIX qw(floor);

# This program specifically checks an HTML file (should be the page with News
# and Events listings on it) for lines containing news and events links.  Such
# lines should be formatted like: <a href="LINKURL">TITLE</a>...<em>DATE</em>
# with some additional constraints.  The link must either point to a full URL or
# to a page in http://bussvc.wisc.edu/ecbs/ and the date must be formatted as the
# standard 3-letter month abbreviation (see the hard-coded list below), followed
# by the day of the month (does NOT require two digits).  Note that this program
# will break in the year 2100, and will need to be updated if still in use at 
# that time.
# 
# Future suggestions: Including the year in the HTML (can handle New Year); find a 
# way to pull a description or some content from somewhere (maybe the linked-to
# document?); automatic publishing to the server via ftp or other;
# integration with some other script or program to automate the entire news/event
# publishing process (edit homepage HTML to add new event, move last event to
# archive, generate new RSS).

# set up filehandle, variables
open FILE, ">feed.xml" or die "Could not open output file!";
my ($link, $title, $date, $latest);
my @dabbr;
my @mabbr;
my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst);
select FILE;
# start the xml file with the rss header and channel information
print "<?xml version=\"1.0\"?>\n";
print "<rss version=\"2.0\" xmlns:atom=\"http://www.w3.org/2005/Atom\">\n";
print "<channel>\n<title>UW-Madison OHR News and Events</title>\n";
print "<link>http://benefits.wisc.edu/</link>\n";
print "<description></description>\n";
print "<language>en-us</language>\n";
print "<atom:link href=\"http://www.bussvc.wisc.edu/ecbs/feed.xml\" rel=\"self\"";
print " type=\"application/rss+xml\" />";
# go through the homepage line by line looking for news items like <a>...</a>...<em>...</em>
while(<>) {
  # get the link, title, and date out of the HTML
  m/<a(.*?)>(.*?)<\/a>.*?<em>(.*?)<\/em>/;
  my $link = $1;
  my $title = $2;
  my $date = $3;
  # extract the link URL
  $link =~ m/href="(.*?)"/;
  $link = $1;
  # if the link used a relative path, change it to absolute
  ($link =~ m/http:\/\//) or $link = "http://www.bussvc.wisc.edu/ecbs/$link";
  # get the current time (we're really only interested in the year)
  ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = 
     localtime(time);
  # set up the abbreviation lists
  @mabbr = qw( Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec );
  @dabbr = qw( Sun Mon Tue Wed Thu Fri Sat );
  # swap the month and day, for standards compliance
  $date =~ s/(\w{3}) (\d{1,2})/$2 $1/;
  # set up for day-of-week calculation
  $mday = $2;
  $year -= 100;
  $mon = 0;
  ++$mon until $mabbr[$mon] eq $1;
  # the first month we see should be the latest one
  $latest = $mon if $latest == undef;
  # otherwise, the date was from last year
  --$year if $mon > $latest;
  $mon--;
  if($1 eq "Jan") {
    $mon = 11;
    $year--;
  } elsif($1 eq "Feb") {
    $mon = 12;
    $year--;
  }
  # calculate which day of the week the date fell on
  $wday = ($2 + floor(2.6*$mon - 0.2) - 40 + $year + floor($year/4) + 5) % 7;
  $wday = $dabbr[$wday];
  # put the year into 4-digit format
  $year += 2000;
  # if we have a valid date, we should put this line's data into the feed
  if($date ne "") {
    # format the date into a nice string
    $date = "$wday, $date $year 00:00:00 CST";
    # write the appropriate XML to file
    print "<item>\n<title>$title</title>\n";
    print "<link>$link</link>\n<pubDate>$date</pubDate>\n";
    print "<guid isPermaLink=\"true\">$link</guid>\n</item>\n";
  }
}
# finish up the XML and clean up
print "<\/channel><\/rss>\n";
select STDOUT;
close FILE;
# validate the feed using feedvalidator
my @valid = `python feedvalidator/src/demo.py feed.xml`;
if(("".@valid == 2) && ($valid[1] =~ m/Self reference/)) {
  # We're okay, print success
  print "Success!\n\n";
} else {
  # Oh no, feed didn't validate!  Throw an error!"
  print "An error occurred and the feed could not be validated!\n\n";
  print @valid;
}
